-- Pakoskyfrog 2013/03/02 19:10:00

-----------------------------------------------------------
----    CGame definition
-----------------------------------------------------------

------------------------
-- Description
--[[
    The is the core of the game 'Memory Glyph'
]]

-----------------------------------------------------------
require 'gameBG'

------------------------
--  Init
CGame = {}
CGame.__index = CGame

------------------------
--  Properties
CGame.type = "CGame"


------------------------
--  Constructor
function CGame:create(w, h)
    local Game = {}
    setmetatable(Game, CGame)
    
    Game.bg = CGameBG:create()
    
    Game.selected = {1,1}
    Game.width  = w or 9
    Game.height = h or 4
    Game.miss   = 0
    Game.found  = 0
    
    -- TODO : filter w/h so there is an even number of cards
    
    Game:load()
    
    print('Game created.')
    return Game
end


------------------------
--  Callbacks

function CGame:load()
    -- cards
    io.write('Generating cards... ')
    self.cards = {}
    
    for c = 1, self.width*self.height*0.5 do
        local card = CGlyph:create()
        local id = card.uid -- should start at 1
        self.cards[id] = card
    end
    io.write('done.\n')
    
    
    -- grid
    io.write('Generating grid... ')
    
    self.grid = {}
    for i = 1, self.height do
        self.grid[i] = {}
        for j = 1, self.width do
            local card
            repeat
                -- this method could be long... for a large set, which we don't have.
                local rc = math.random(#self.cards)
                card = self.cards[rc]
            until card.taken < 2
            
            card.taken = card.taken + 1
            -- card.discovered = true
            self.grid[i][j] = card.uid
        end
    end
    io.write('done.\n')
    
    self.tob = 0
    self.timer = 0
end

function CGame:draw()
    -- background
    if self.bg then
        self.bg:draw()
    end

    -- grid
    local ox, oy = 15, 50
    local dx, dy = 12, 15
    for i = 1, self.height do
        local y = (CGlyph.height+dy) * (i-1)
        for j = 1, self.width do
            local card = self.cards[self.grid[i][j]]
            
            if self.place1 and (self.place1[1]==j and self.place1[2]==i) or self.place2 and (self.place2[1]==j and self.place2[2]==i) then
                card.discovered = true
            end
            
            local x = (CGlyph.width+dx) * (j-1)
            -- if not card.inPair then -- not found already
                card:draw(x+ox, y+oy)
            -- end
            
            if self.place1 and (self.place1[1]==j and self.place1[2]==i) or self.place2 and (self.place2[1]==j and self.place2[2]==i) then
                card.discovered = false
            end
        end
    end
    
    -- selected :
    do
        local x = (CGlyph.width +dx) * (self.selected[1]-1) - 2 + ox
        local y = (CGlyph.height+dy) * (self.selected[2]-1) - 2 + oy
        love.graphics.setColor(255,127,127)
        love.graphics.setLineWidth(4)
        love.graphics.rectangle('line', x,y, CGlyph.width+4, CGlyph.height+4)
    end
    
    -- time elapsed :
    love.graphics.setColor(64,64,127)
    do
        local M = math.floor(self.timer/60)
        local S = math.floor(self.timer - M*60)
        local s = (self.tob%1)>0.5 and ':' or '.'
        love.graphics.setFont(Apps.fonts.big)
        love.graphics.print(M..s..S, 12,5)
    end
    
    -- errors made
    love.graphics.setColor(127,64,64)
    do
        love.graphics.setFont(Apps.fonts.big)
        love.graphics.print(tostring(self.miss), 150,5)
    end
    
    -- founds
    love.graphics.setColor(64,127,64)
    do
        love.graphics.setFont(Apps.fonts.big)
        love.graphics.print(tostring(self.found), 300,5)
    end
    
    -- win
    if self.win then
        local w = love.graphics.getWidth()
        local h = love.graphics.getHeight()
        local ds = 250
        local sw = w - 2*ds
        local sh = h - 2*ds
        love.graphics.setColor(15,127,20,200)
        love.graphics.rectangle('fill', ds,ds, sw,sh)
        love.graphics.setFont(Apps.fonts.giant)
        local mx = 0.5 * Apps.fonts.giant:getWidth('Win !')
        local my = 0.5 * Apps.fonts.giant:getHeight()
        love.graphics.setColor(127+math.sin(self.tob)*127, 127+math.sin(self.tob+2*math.pi/3)*127, 127+math.sin(self.tob+4*math.pi/3)*127)
        love.graphics.print('Win !', w*0.5-mx, h*0.5-my)
        love.graphics.setLineWidth(5)
        love.graphics.rectangle('line', ds,ds, sw,sh)
    end
end

function CGame:update(dt)
    -- background
    if self.bg then
        self.bg:update(dt)
    end

    self.tob = self.tob + dt
    if self.win then return end
    
    self.timer = self.timer + dt
    
    if self.hideTime then
        self.hideTime = self.hideTime - dt
        if self.hideTime < 0 then
            self.place1 = nil
            self.place2 = nil
            
            self.card1.inPair = self.inPair
            self.card2.inPair = self.inPair
            
            self.card1 = nil
            self.card2 = nil
            self.hideTime = nil
            self.inPair = nil
        end
    end
    
    -- simple framerate limiter
    if dt < 0.016 then
        love.timer.sleep(0.016 - dt)
    end
end

function CGame:mousepressed(x, y, btn)
    if self.hideTime then return end
    
end

function CGame:keypressed(key)
    if key == 'escape' then
        CGlyph.uid = 0
        Apps.game = nil
    end
    
    if not self.win then -- old ugly gamestate
        if key == 'left' then
            self.selected[1] = (self.selected[1]-1 -1) %self.width  +1
        elseif key == 'right' then
            self.selected[1] = (self.selected[1]+1 -1) %self.width  +1
        elseif key == 'up' then
            self.selected[2] = (self.selected[2]-1 -1) %self.height +1
        elseif key == 'down' then
            self.selected[2] = (self.selected[2]+1 -1) %self.height +1
        elseif (key == ' ' or key == 'return') and (not self.hideTime) then
            self:pickup()
        end
    end
end

function CGame:mousereleased(x, y, btn)
    
end

function CGame:keyreleased(key)
    
end

------------------------
--  Static functions


------------------------
--  Member functions
function CGame:getType()
    return self.type
end

function CGame:pickup()
    if not self.card1 then
        self.card1 = self.cards[self.grid[self.selected[2]][self.selected[1]]]
        if self.card1.inPair then
            self.card1 = nil
            return
        end
        self.place1 = {unpack(self.selected)}
        print('1:', self.card1.uid)
    elseif not self.card2 and not(self.place1[1]==self.selected[1] and self.place1[2]==self.selected[2]) then
        self.card2 = self.cards[self.grid[self.selected[2]][self.selected[1]]]
        if self.card2.inPair then
            self.card2 = nil
            return
        end
        self.place2 = {unpack(self.selected)}
        print('2:', self.card2.uid)
    end
    
    if self.card1 and self.card2 then
        self.hideTime = 0.75
        if self.card1.uid == self.card2.uid then
            -- found a match !
            print('pop')
            self.inPair = true
            self.found = self.found + 1
            
            if self.found == self.width*self.height*0.5 then
                print('youhou !!!')
                self.win = true
            end
        else
            self.miss = self.miss + 1
        end
    end
end

-- function CGame:hasWon()
    -- --------------------
    -- --  Check if you've won
    -- local test = true
    -- for i, c in ipairs(self.cards) do
        -- test = test and c.inPair
    -- end
    -- return test
-- end


print "CGame loaded"