-- Pakoskyfrog 2013/03/05 16:36:40

-----------------------------------------------------------
----    CGameBG definition
-----------------------------------------------------------

------------------------
-- Description
--[[
    This is the background of the game.
    It displays a glyph going from random to smooth.
]]

-----------------------------------------------------------
------------------------
--  Init
CGameBG = {}
CGameBG.__index = CGameBG

------------------------
--  Properties
CGameBG.type = "CGameBG"
CGameBG.tic = 0.02  -- the glyph is actualized every this amount of seconds
CGameBG.hiden = 150 -- this is the number of hiden iterations done before displaying
CGameBG.stop = 1000 -- this is the number max of iterations
CGameBG.iter = 0    -- this is the number iterations currently done
CGameBG.step = 1    -- this is the number of iterations to do on each tic
CGameBG.N = 200     -- number of points in the glyph

------------------------
--  Constructor
function CGameBG:create()
    local GameBG = {}
    setmetatable(GameBG, CGameBG)
    
    GameBG.color = self.newColor()
    GameBG.timer = 0
    
    -- GameBG.pointList = {}
    
    GameBG:newGlyph()
    
    print('Game BG created.')
    return GameBG
end


------------------------
--  Callbacks
function CGameBG:draw()
    love.graphics.setColor(self.color)
    love.graphics.setLineWidth(2)
    love.graphics.line(self.pointList)
end

function CGameBG:update(dt)
    self.timer = self.timer + dt
    if self.timer > self.tic then
        self:onTic()
        self.timer = self.timer - self.tic
    end
end

------------------------
--  Static functions
function CGameBG.newColor()
    --------------------
    --  this generates a new bright color
    return {math.random(205)+50, math.random(205)+50, math.random(205)+50}
end

function CGameBG.randomPairList(N)
    --------------------
    --  this will generate a list of random numbers 0..1
    -- of length = 2*N
    
    local list = {}
    for i = 1, N do
        table.insert(list, math.random())
        table.insert(list, math.random())
    end
    return list
end


------------------------
--  Member functions
function CGameBG:getType()
    return self.type
end

function CGameBG:onTic()
    --------------------
    --  this handles the iterations
    
    for i = 1, self.step do
        -- iter...
        self:scale(self:mean())
    end
    self.iter = self.iter + self.step
    
    if self.iter > self.stop then
        self:newGlyph()
    end
end

function CGameBG:newGlyph()
    --------------------
    --  This will create a whole new glyph
    
    -- define a new color
    self.color = self.newColor()
    
    -- initial random number list
    self.pointList = CGameBG.randomPairList(self.N)
    
    -- do the hiden iterations :
    self.iter = 0
    for i = 1, self.hiden-1 do
        self:mean()
    end
    local mx,my, Mx,My = self:mean()
    print(mx,my, Mx,My)
    self.iter = self.hiden
    
    -- Scaling :
    self:scale(mx,my, Mx,My)
    -- print(self:mean())
end

function CGameBG:scale(mx,my, Mx,My)
    --------------------
    --  this will scale the pointList to fit in screen
    
    local h = math.min(love.graphics.getHeight(), love.graphics.getWidth())
    local sx = (love.graphics.getWidth()-h) * 0.5 -- assuming width is larger than height
    local factor = h / math.max(Mx-mx, My-my)
    local pts = self.pointList
    for i = 1, self.N do
        local xi = (i-1)*2 + 1
        local yi = i * 2
        pts[xi] = (pts[xi]-mx) * factor + sx
        pts[yi] = (pts[yi]-my) * factor
    end
    
end

function CGameBG:mean()
    --------------------
    --  this will get the mean points of every points in self.pointList
    -- circular condition : the last and the first are connected
    -- this function returns mx,my, Mx,My
    
    local inf = math.huge
    local mx,my, Mx,My = inf,inf, -inf,-inf
    
    local pts = self.pointList
    
    for i = 1, self.N do
        local i2 = i+1
        if i2 > self.N then i2 = 1 end
        
        -- P1(x1,y1), P2(x2,y2)
        local x1 = pts[(i-1)*2 + 1]
        local y1 = pts[i * 2]
        local x2 = pts[(i2-1)*2 + 1]
        local y2 = pts[i2 * 2]
        
        -- M = (P1+P2)/2
        local x = (x1+x2)*0.5
        local y = (y1+y2)*0.5
        
        pts[(i-1)*2 + 1] = x
        pts[i * 2] = y
        
        mx = math.min(mx, x)
        my = math.min(my, y)
        Mx = math.max(Mx, x)
        My = math.max(My, y)
    end
    
    return mx,my, Mx,My
end


print "CGameBG loaded"