-- Pakoskyfrog 2013/03/02 19:08:23

-----------------------------------------------------------
----    CGlyph definition
-----------------------------------------------------------

------------------------
-- Description
--[[
    This is a card of the game, at its creation the glyph inside is auto generated.
]]

-----------------------------------------------------------
------------------------
--  Init
CGlyph = {}
CGlyph.__index = CGlyph

------------------------
--  Properties
CGlyph.type = "CGlyph"
CGlyph.uid = 0
CGlyph.height = 150
CGlyph.width = 100
CGlyph.discovered = false
CGlyph.inPair = false
CGlyph.taken = 0
CGlyph.N = 75 -- nbr of points in the glyph symbol

------------------------
--  Constructor
function CGlyph:create()
    local Glyph = {}
    setmetatable(Glyph, CGlyph)
    
    Glyph.setUID(Glyph)
    Glyph.color = {math.random(205)+50, math.random(205)+50, math.random(205)+50}
    Glyph.pts = self.generatePoints()
    
    return Glyph
end


------------------------
--  Callbacks
function CGlyph:draw(x,y)
    love.graphics.setColor(0,0,0, 200)
    love.graphics.rectangle('fill', x,y, self.width, self.height)
    
    if not self.inPair then
        love.graphics.setColor(127,255,127)
        love.graphics.setLineWidth(2)
    else
        love.graphics.setColor(127,255,127,50)
        love.graphics.setLineWidth(1)
    end
    love.graphics.rectangle('line', x,y, self.width, self.height)
    
    if self.discovered or self.inPair then
        -- love.graphics.setColor(127,127,255)
        -- love.graphics.setFont(Apps.fonts.normal)
        -- love.graphics.print(tostring(self.uid), x+10, y+10)
        
        love.graphics.setColor(self.color)
        love.graphics.setLineWidth(2)
        love.graphics.push()
        love.graphics.translate(x+self.width*0.5, y+self.height*0.5)
        love.graphics.line(self.pts)
        love.graphics.pop()
    end
end

------------------------
--  Static functions
function CGlyph.setHeight(h)
    CGlyph.height = h
end
function CGlyph.setWidth(w)
    CGlyph.width = w
end

function CGlyph.generatePoints()
    --------------------
    --  This is the core of the game : 
    -- here, we generate the different glyphs
    
    local xx = {}
    local yy = {}
    
    local N = CGlyph.N
    
    local rd = function ()
        -- -1..1
        return math.random()*2-1
    end
    local meanPoint = function (x1, y1, x2, y2, n)
        local xm = (x1+x2)*0.5 *n
        local ym = (y1+y2)*0.5 *n
        return xm, ym
    end
    
    -- 1st gen
    for i = 1, N do
        xx[i] = rd()
        yy[i] = rd()
        -- print('('..xx[i]..':'..yy[i]..')')
    end
    
    -- smoothing
    local mx,my, Mx,My -- maxes & mins
    local norm = 1 -- renormalization factor
    for iter = 1, 200 do
        mx,my, Mx,My = 0.5,0.5,0.5,0.5
        for i = 1, N do
            local i1 = i
            local i2 = (i) % N + 1 -- looping
            xx[i1], yy[i1] = meanPoint(xx[i1], yy[i1], xx[i2], yy[i2], norm)
            
            mx = math.min(mx, xx[i1])
            my = math.min(my, yy[i1])
            Mx = math.max(Mx, xx[i1])
            My = math.max(My, yy[i1])
        end
        norm = 1/math.max(Mx,-mx, My,-my)
    end
    
    -- interleaving
    local out = {}
    local k = (CGlyph.width*0.4) * norm
    for i = 1, N do
        out[2*(i-1)+1] = xx[i] * k
        out[2*(i-1)+2] = yy[i] * k
    end
    -- close the loop
    out[2*N+1] = out[1]
    out[2*N+2] = out[2]
    return out
end


------------------------
--  Member functions
function CGlyph:getType()
    return self.type
end

function CGlyph:setUID()
    CGlyph.uid = CGlyph.uid + 1
    self.uid = CGlyph.uid
end
function CGlyph:getUID()
    return self.uid
end





print "CGlyph loaded"