-- ========================================================
-- ========         Memory Glyph,              ============
-- ========         Pakoskyfrog 2013/03/02     ============
-- ========================================================

------------------------
--  Description
--[[
    This is a simple (mini)game where you have to find matching pairs.
    There is no pictures, you have to match generated glyphs.
    
    Gamestates and controls are minimal for now.
]]

------------------------
--  TODO list
--[[
    * win conditions
    * add number of errors made
    * add mouse interaction in game
    * animation when a pair is found
    * animation to turn over a card
]]


------------------------
--  Dependencies
require 'glyph'
require 'game'

------------------------
--  Init
Apps = {}
Apps.version = 0.1

Apps.fonts = {
    mini   = love.graphics.newFont(10),
    title  = love.graphics.newFont(50),
    normal = love.graphics.newFont(24),
    big    = love.graphics.newFont(33),
    giant  = love.graphics.newFont(110),
}

local menuSelected = 1
local menuCmds = {
    -- {'text to display', 'key to trigger'}
    {'Launch the game', 'g'},
    {'Have a free cookie', 'k'},
    {'Quit', 'escape'}
}

------------------------
--  Callbacks
function love.draw()
    if Apps.game then
        -- in game
        Apps.game:draw()
    else
        -- in menu
        drawMenu()
    end
    
    love.graphics.setColor(0,127,0, 127)
    love.graphics.setFont(Apps.fonts.mini)
    local w,h = love.graphics.getWidth(), love.graphics.getHeight()
    love.graphics.print('Pakoskyfrog 2013', w-100, h-15)
end
function drawMenu()
    love.graphics.setColor(127,255,127)
    love.graphics.setFont(Apps.fonts.title)
    love.graphics.print('Memory glyphs v'..Apps.version, 5, 5)
    
    love.graphics.setColor(127,127,255)
    love.graphics.setFont(Apps.fonts.normal)
    
    for l = 1, #menuCmds do
        local text = menuCmds[l][1]
        local trigger = menuCmds[l][2]
        love.graphics.print('['..trigger..'] : '..text, 50, 75 + l*30)
    end
end

function love.keypressed(key)
    if Apps.game then
        -- in game
        Apps.game:keypressed(key)
    else
        -- in menu
        keyMenu(key)
    end
end
function keyMenu(key)
    if key == 'escape' then
        love.event.push('quit')
    elseif key == 'g' then
        launchGame()
    elseif key == 'k' then
        print('There is some in the kitchen !')
        print('Did you know that 1733^2 = 3003289 ?')
    else
        -- wrong key
    end
    
end

function love.update(dt)
    if Apps.game then Apps.game:update(dt) end
end

------------------------
--  Menu Actions

function launchGame()
    Apps.game = CGame:create()
end
